;This is the assembler code for the PICROCK project
;Willem Deconink
;Bram Verkens
;4 WE/WB
;2005

;motor1 moves the plectrum
;motor2 moves the roll
;motor3 rotates the plectrum
;motor4 moves the weight up and down

	LIST   P=PIC16F877a
	#INCLUDE P16F877a.INC
	errorlevel -302		;suppress "not in bank 0" message

	__CONFIG        _CP_OFF & _DEBUG_OFF & _LVP_OFF & _PWRTE_ON & _WDT_OFF & _BODEN_OFF & _HS_OSC

	CBLOCK	0x20
        WREG_TEMP		;storage for WREG during interrupt
	STATUS_TEMP		;storage for STATUS during interrupt
	PCLATH_TEMP		;storage for PCLATH during interrupt
	FSR_TEMP		;storage for FSR during interrupt

	DIRECTION		;information on rotation direction
	POWERLEVEL		;information of which current to use
	MOTORSTEPS1
	MOTORSTEPS2
	MOTORSTEPS1_1		;number of cycles of 256 steps (motor1)
	MOTORSTEPS1_2		;number of steps that we have to do extra (motor1)
	MOTORSTEPS2_1		;(motor2)
	MOTORSTEPS2_2		;(motor2)
	MOTORSTEPS3_1		;(motor3)
	MOTORSTEPS3_2		;(motor3)
	MOTORSTEPS4_1		;(motor4)
	MOTORSTEPS4_2		;(motor4)
	STEPPER			;Which stepper are we going to make work?

	CHECKMOTORS
	
	TIMERDELAY1		;timerdelayX are help variables for motor X, they help determine Timertellerx
	TIMERDELAY3
	TIMERDELAY4	
	TIMERTELLER1		;timertellerX determine the wait time between two steps using timer interrupts
	TIMERTELLER3
	TIMERTELLER4
	
	ENABLE			;byte to check if we should enable or disable the controlchips L297
			
	INITIALISE		;byte to check which motor we should initialise, or if the initialisation is already done
				;this determines if data is send to the pc when a switch is pressed or not

	STEPS		

	STEPS1			;stepsX help variable for the loops of 256 steps
	STEPS2
	STEPS3
	STEPS4

	DATAIN1			;Received data
	DATAIN2			;Received data
	DATAIN3			;Received data
	DATAIN4			;Received data
        DATAIN5			;Received data
	DATAIN6			;Received data
	DATAIN7			;Received data
	DATAIN8			;Received data
	ENDC
;----------------------------------------------------------------------------
;RESET Interrupt Vector
		ORG     0x0000		;reset vector
 		GOTO	Main
		nop
		nop
;----------------------------------------------------------------------------
;Other interrupt vector
		ORG	0x0004		;interrupt vector
		GOTO 	Int

;******************************************************************************
;******************************************************************************

Main:		;first we make all the needed ports into in-or outputs
		BSF	STATUS,RP0  ;bank 1
		BCF	TRISC,1
		BCF	TRISC,2
		MOVLW	B'00001100' ;D1, D6, D7 output, D3, D2 input
		MOVWF	TRISD
                MOVLW   B'00000100' ;A5, A4, A3, A1, A0 output, A2 input
		MOVWF   TRISA
		MOVLW	B'00001001' ;B1, B2, B4, B5 output, B0, B3 input
		MOVWF	TRISB
		MOVLW	B'00111000' ;C3, C4,C5 input , C2 output

		BCF	STATUS,RP0 	;bank0
		CALL	SetupSerial	;define the serial connection
		CALL	SetupDigital	;make all outputs of PORTA digital
		BCF	STATUS,RP0
		MOVLW	B'00000000'
		MOVWF	CHECKMOTORS	;no motors should be working

		BSF	PORTC,2		;DISABLE RESET
		BSF	PORTA,4		;ENABLE MOTORS
		BSF	INTCON,GIE	;ENABLE ALL INTERRUPTS
		
		MOVLW	B'00011111'	;PRETEND THE SYSTEM IS INITIALISED
		MOVWF	INITIALISE	

MAINLOOP
		Bank0		
		BTFSS	PORTC,5		;check if switch 1 is pressed
		GOTO	$+5
		BTFSC	INITIALISE,1	;check if motor 1 needs to be initialised
		GOTO	$+3
		MOVLW	B'01100100'	;we just write 100 to the pc when MicroSwitch 1 is pressed
		MOVWF	TXREG

		BTFSS	PORTC,4		;check if switch 2 is pressed
		GOTO	$+5
		BTFSC	INITIALISE,2	;check if motor 2 needs to be initialised
		GOTO	$+3
		MOVLW	B'01111000'	;we just write 120 to the pc when MicroSwitch 2 is pressed
		MOVWF	TXREG

		BTFSS	PORTD,3		;check if switch 3 is pressed
		GOTO	$+9	
		BTFSC	INITIALISE,3	;check if motor 3 needs to be initialised
		GOTO	$+7
		MOVLW	B'10001100'	;we just write 140 to the pc when MicroSwitch 3 is pressed
		MOVWF	TXREG
		MOVLW	B'00000000'
		MOVWF	TXREG		;we manually clear the transmitter register
		MOVLW	B'00011111'	;we say the system is initialised here
		MOVWF	INITIALISE
		
		BTFSS	PORTD,3		;we write 112 to the pc when microswitch 3 is pressed and the system is initialised
		GOTO	$+5		;this means the plectrum has to stop moving.
		BTFSS	INITIALISE,5	;this should happen only when the fifth bit of the initialise is set
		GOTO	$+3
		MOVLW	B'01110000'	;send 112 to the PC
		MOVWF	TXREG

		GOTO 	MAINLOOP

;******************************************************************************
;******************************************************************************
SetupSerial:	BSF	STATUS,RP0			;select bank 1
		BCF	TRISC,6		; Make RC6 - TX pin an output
		BSF	TRISC,7		; Make RC7 - RX pin an input
		BCF	TXSTA,BRGH	;Select low speed mode
		MOVLW	D'12'
		MOVWF	SPBRG		;Select baud rate 9600 in low speed (8MHz crystal)
		BCF	TXSTA,SYNC	;Select async mode
		BSF	TXSTA,TXEN	;Enable Transmit

		BCF	STATUS,RP0			;select bank0
		movlw	0x90		;enable serial port and reception
		movwf	RCSTA
		movlw	0xc0		;enable global and peripheral ints
		movwf	INTCON
		BSF	STATUS,RP0			;select bank1
		movlw	0x20		; (0x30 enable TX) and RX interrupts
		movwf	PIE1

		BSF	PIE1,TMR1IE	;enable TIMER1 interrupt
					;TIMER1 will determine the time between two pulses to the controlchip
					;every pulse means one step, the value of timer1 goes down when the motor
					;has made a step. That way the motor will accelerate until a maximal speed.
					;timer1 is a 16-bit timer and is used for longer times.
		BSF	PIE1,TMR2IE	;enable TIMER2 interrupt
					;Timer2 determines the length of a pulse that is send to the controlchip
					;timer2 is an 8-bit timer and is used for short times
		BCF	STATUS,RP0
		Bank0
		BCF	T1CON,TMR1CS	;use internal clock for timer
		BSF	T1CON,TMR1ON	;turn timer1 on, every timerinterrupt, the program will check which motor to move

		MOVLW	B'01100100'	;put 100->256 as delay for TIMER2, that's the short time for the length of the pulse
		MOVWF	TMR2		;TMR2 is the starting number of the TIMER2 (100 in this case)

		return

SetupDigital:
		BSF	STATUS,RP0
		MOVLW	B'01000111'     ;Makes all analog ports of A digital
		MOVWF	ADCON1
		BCF	STATUS,RP0
		MOVLW	B'01000000'
		MOVWF	ADCON0
		RETURN

;******************************************************************************
;******************************************************************************
Int:		movwf	WREG_TEMP	;save WREG
		movf	STATUS,W	;store STATUS in WREG
		clrf	STATUS		;select file register bank0
		movwf	STATUS_TEMP	;save STATUS value
		movf	PCLATH,W	;store PCLATH in WREG
		movwf	PCLATH_TEMP	;save PCLATH value
		clrf	PCLATH		;select program memory page0
		movf	FSR,W		;store FSR in WREG
		movwf	FSR_TEMP	;save FSR value

		Bank0
		BTFSS	PIR1,TMR1IF	;Test for Timer1 interrupt flag
		GOTO	$+3
		BCF	PIR1,TMR1IF
		GOTO	TimInt		;;If TIMER1 is the interrupt, go to this procedure

		Bank0
		BTFSS	PIR1,TMR2IF	;Test for TIMER2 interrupt flag
		GOTO	$+3
		BCF	PIR1,TMR2IF
		GOTO	Tim2Int		;If TIMER2 is the interrupt, go to this procedure

		BCF	STATUS,RP0	;select bank0
		btfsc	PIR1,RCIF	;test RCIF receive interrupt
		bsf	STATUS,RP0	;change to bank1 if RCIF set
		btfsc	PIE1,RCIE	;test if interrupt enabled if RCIF set
		goto	GetData		;if RCIF and RCIE set, do receive

		BCF	STATUS,RP0	;select bank0
		btfsc	PIR1,TXIF	;test for TXIF transmit interrupt
		bsf	STATUS,RP0	;change to bank1 if TXIF set
		btfsc	PIE1,TXIE	;test if interrupt enabled if TXIF set
		goto	PutData		;if TXIF and TCIE set, do transmit

;can do special error handling here - an unexpected interrupt occurred
EndInt:		BCF	STATUS,RP0			;select bank 0
		movf	FSR_TEMP,W	;get saved FSR value
		movwf	FSR		;restore FSR
		movf	PCLATH_TEMP,W	;get saved PCLATH value
		movwf	PCLATH		;restore PCLATH
		movf	STATUS_TEMP,W	;get saved STATUS value
		movwf	STATUS		;restore STATUS
		swapf	WREG_TEMP,F	;prepare WREG to be restored
		swapf	WREG_TEMP,W	;restore WREG without affecting STATUS
		;GOTO	MAINLOOP
		retfie			;return from interrupt

;******************************************************************************
;******************************************************************************
;Get received data
GetData:	BCF	STATUS,RP0
		btfsc	RCSTA,OERR	;test overrun error flag
		goto	ErrOERR		;handle it if error
		btfsc	RCSTA,FERR	;test framing error flag
		goto	ErrFERR		;handle it if error
				

		movf	DATAIN7,W	;Move last received data
		movwf	DATAIN8

		movf	DATAIN6,W	;Move last received data
		movwf	DATAIN7
		
		movf	DATAIN5,W	;Move last received data	
		movwf   DATAIN6

                movf	DATAIN4,W	;Move last received data
		movwf	DATAIN5

		movf	DATAIN3,W	;Move last received data
		movwf	DATAIN4

		movf	DATAIN2,W	;Move last received data
		movwf	DATAIN3

		movf	DATAIN1,W	;Move last received data
		movwf	DATAIN2

		movf	RCREG,W		;Get received data
		movwf	DATAIN1		; Copy to DataIn1

		SUBLW	D'0'
		BTFSS	STATUS,Z	; test if last data was 0
		GOTO	BlockNotComplete

		BCF	STATUS,RP0
		movf    DATAIN5,w
		movwf	MOTORSTEPS1	;Move data to MOTORSTEPS1

		movf	DATAIN2,W	;Move data to POWERLEVEL
		movwf	POWERLEVEL

		movf	DATAIN3,W	;Move data to DIRECTION
		movwf	DIRECTION

		movf	DATAIN4,W	;Move data to MOTORSTEPS2
		MOVWF	MOTORSTEPS2

		movf    DATAIN6,W	;Move data to STEPPER
		movwf	STEPPER

		movf	DATAIN7,W	;Move data to ENABLE
		MOVWF	ENABLE

		movf	DATAIN8,W	;Move data to INITIALISE
		MOVWF	INITIALISE

		BTFSS	ENABLE,2	;bit 2 determines if the controlchips will be resetted or not
		BCF	PORTC,2		;RESET

		BTFSC	ENABLE,2	;bit 2 determines if the controlchips will be resetted or not
		BSF	PORTC,2		;Don't RESET

		BTFSS	ENABLE,0	;bit 0 determines if the controlchips are enabled or not
		BSF	PORTA,4		;ENABLE

		BTFSC	ENABLE,0	;bit 0 determines if the controlchips are enabled or not
		BCF	PORTA,4		;DISABLE

		BTFSC   STEPPER,1
		GOTO	MOTOR1_INIT	;initialise all motorconstants for motor1

		BTFSC   STEPPER,2
		GOTO	MOTOR2_INIT	;initialise all motorconstants for motor2

		BTFSC 	STEPPER,3	;initialise all motorconstants for motor3
		GOTO	MOTOR3_INIT	
	
		BTFSC 	STEPPER,4	;initialise all motorconstants for motor4
		GOTO	MOTOR4_INIT
		
		GOTO	EndInt

;motor_INIT initialises all values for the motor when it has to work

MOTOR1_INIT	BSF	CHECKMOTORS,0	;This way the program knows motor 1 has to work

		MOVLW	B'01111000'	;timerconstants,start value for the timer
		MOVWF	TMR1H		;first 8 bits of TIMER1
		MOVLW	B'00000000'
		MOVWF	TMR1L		;second 8 bits of TIMER1

		BTFSC 	POWERLEVEL,1
		BSF	PORTA,0		;high current in stepper 1	
		BTFSS 	POWERLEVEL,1
		BCF	PORTA,0		;low current in stepper 1

		BTFSC 	DIRECTION,1
		BSF	PORTA,1		;clockwise turning
		BTFSS 	DIRECTION,1
		BCF	PORTA,1		;counter-clockwise turning

		;Number of loops of 256 steps.
		movlw	B'00000001'
		SUBWF	MOTORSTEPS1,F
		;Number of extra steps
		movlw	B'00000001'
		SUBWF	MOTORSTEPS2,F
		
		MOVF	MOTORSTEPS1,W	
		MOVWF	MOTORSTEPS1_1	;write away the MOTORSTEPS1 into the proper variable for stepper1
		MOVF	MOTORSTEPS2,W
		MOVWF	MOTORSTEPS1_2	;write away the MOTORSTEPS1 into the proper variable for stepper1

		MOVF	TMR1H,W
		MOVWF	TIMERDELAY1	;write away the starting time in a temp variable
		MOVF	TIMERDELAY1,W
		MOVWF	TIMERTELLER1	;write away the starting time in the real variable	

		MOVLW	B'00000000'
		MOVWF	STEPS1		;steps1 is the number of steps that have to be made still in a loop of 255 steps	

		GOTO	EndInt

;motor_out is ran when the motor stops moving

MOTOR1_OUT	BCF	CHECKMOTORS,0	; motor1 doesn't have to make steps anymore
		BCF	PORTA,0		; make low current again
		RETLW	0
		
MAKESTEPS_1	MOVLW	D'255'
		MOVWF	STEPS1		;put 255 in STEPS1, this is the initialisation for the number of steps to be made still
					;in the loop where 255 steps have to be made.
		RETLW	0


;MotorLong puts the pulse high
;MotorKort puts the pulse low again
;a combination of motorlong and motorkort gives 1 step
;motorlong also accelerates the interrupt if needed

MOTOR1LONG:	Bank0

		BTFSC	CHECKMOTORS,0
		BSF	PORTA,3		;Make pulse high (=clock)
		BTFSC	CHECKMOTORS,1
		BSF	PORTB,4

		movlw	B'00000001'	;ADD 1 to the TMR1H=acceleration
		ADDWF	TIMERTELLER1,F	;Save in workregister to check if subtract gives a 0
		MOVLW	B'11010111'	
		SUBWF	TIMERTELLER1,W	;We subtract 215 to see if we ahave 200 for TMR1H (max speed)
		BTFSC	STATUS,Z	;GOTO next line if TMR1H = 215 (Z = 1)
		GOTO	$+3		 
		movf	TIMERTELLER1,W	;If TMR1H is not 215 we still have to add 1, otherwise we leave TIMERDELAY as it was
		movwf	TIMERDELAY1	;Save subtraction into TIMERDELAY, which is used to determine delay between steps

		MOVF	TIMERDELAY1,W	;write away the timervalues into the proper registers
		MOVWF	TMR1H		;first 8 bits
		MOVLW	B'00000000'
		MOVWF	TMR1L		;second 8 bits

		MOVF	TIMERDELAY1,W
		MOVWF	TIMERTELLER1	;write away to the real variable

		BSF	T2CON,TMR2ON	;turn timer2 on (length of pulse)
		BCF	T1CON,TMR1ON	;turn timer1 off

MOTOR1KORT:	Bank0
		
		BCF	T2CON,TMR2ON	;turn timer2 off, end of pulse
		BSF	T1CON,TMR1ON	;turn timer 1 on (next movement)

		BTFSS	CHECKMOTORS,0	;if motor1 is not on we go to PART2 which is the procedure for motor2
		GOTO	PART2

		MOVLW	B'00000000'
		SUBWF	STEPS1,F
		BTFSC	STATUS,Z	;check if STEPS1 = 0
		CALL	MAKESTEPS_1	;if so , make STEPS1 = 255 

		MOVLW	B'00000000'
		SUBWF	MOTORSTEPS1_1,F
		BTFSS	STATUS,Z	;check if MOTORSTEPS1_1 = 0
		GOTO 	XLOOP1		;if not then run XLOOP1, this is the number of loops with 255 steps
		GOTO	YLOOP1		;if so, then run YLOOP1, this is the number of extra steps.

		
XLOOP1		BCF	PORTA,3		;pull pulse low (= clock)
		
		DECF	STEPS1,F
		BTFSC	STATUS,Z	;Skip next line if Z = 0, this means STEPS is NOT 0
		DECF	MOTORSTEPS1_1,F	;If STEPS = 0 then we have to subtract one of MOTORSTEPS1 (Z = 1)

;		MOVLW	B'00000001'	;we just write 1 to the pc, to count the steps,
;		MOVWF	TXREG		;this is not done in the final version, it slows down too much
		

		MOVLW	B'00000000'
		SUBWF	MOTORSTEPS1_1,W
		BTFSS	STATUS,Z	;check if motorsteps1_1 = 0
		GOTO	$+5		;if not we go to part2 and check motor2
		MOVLW	B'00000000'
		SUBWF	MOTORSTEPS1_2,W	;if it is, we check if motorsteps1_2 = 0
		BTFSC	STATUS,Z
		GOTO	MOTOR1_OUT	;if it is that means motor1 doesn't have to make anymore moves

		GOTO	PART2		

YLOOP1   	BCF	PORTA,3		;pull pulse low (= clock)
					
		MOVLW	B'00000001'	;we just write 1 to the pc, to count the steps,
;		MOVWF	TXREG		;this is not done in the final version, it slows down too much

		DECFSZ	MOTORSTEPS1_2,F ;we have made a step, so the number of steps to make goes down with 1
		GOTO	PART2		;if not 0 we check motor2
		
		MOVLW	B'11111111'     ;we send 255 when MOTOR1 finishes it's number of steps
		MOVWF	TXREG		;and transmit

		CALL	MOTOR1_OUT	;motor 1 has made all needed steps, so we run the off function
		GOTO	PART2		;we check motor2

;part2 controls motor2

PART2		BTFSS	CHECKMOTORS,1	;if motor2 doesn't need to make steps, the interrupt ends
		GOTO	EndInt

		MOVLW	B'00000000'
		SUBWF	STEPS2,F	;check if STEPS2 = 0
		BTFSC	STATUS,Z
		CALL	MAKESTEPS_2	;if so , make STEPS2 = 255 

		MOVLW	B'00000000'
		SUBWF	MOTORSTEPS2_1,F	;check if MOTORSTEPS2_1 = 0
		BTFSS	STATUS,Z
		GOTO	XLOOP2		;if not then run XLOOP2, this is the number of loops with 255 steps
		GOTO	YLOOP2		;if so, then run YLOOP2, this is the number of extra steps.
		
XLOOP2		BCF	PORTB,4		;pull pulse low (= clock)
		
		DECF	STEPS2,F
		BTFSC	STATUS,Z	;Skip next line if Z = 0, this means STEPS is NOT 0
		DECF	MOTORSTEPS2_1,F	;If STEPS = 0 then we have to subtract one of MOTORSTEPS1 (Z = 1)

;		MOVLW	B'00000010'	;we just write 2 to the pc, to count the steps,
;		MOVWF	TXREG		;this is not done in the final version, it slows down too much

		MOVLW	B'00000000'
		SUBWF	MOTORSTEPS2_1,W	
		BTFSS	STATUS,Z	;check if motorsteps2_1 = 0
		GOTO	$+5		;if not the interrupt ends
		MOVLW	B'00000000'
		SUBWF	MOTORSTEPS2_2,W	;if it is, we check if motorsteps2_2 = 0
		BTFSC	STATUS,Z
		GOTO	MOTOR2_OUT	;if it is that means motor2 doesn't have to make anymore moves

		GOTO	EndInt		;interrupt ends


YLOOP2   	BCF	PORTB,4		;pull pulse low (= clock)
					
;		MOVLW	B'00000010'	;we just write 2 to the pc, to count the steps,
;		MOVWF	TXREG		

		DECFSZ	MOTORSTEPS2_2,F	;we have made a step, so the number of steps to make goes down with 1
		GOTO	EndInt		;if not 0 the interrupt ends

		
		MOVLW	B'01111111'     ;we send 127 when MOTOR2 finishes it's number of steps
		MOVWF	TXREG		;and transmit

		CALL	MOTOR2_OUT	;motor 2 has made all needed steps, so we run the off function
		GOTO	EndInt



;Now MOTOR2


;motor_INIT initialises all values for the motor when it has to work

MOTOR2_INIT	BSF	CHECKMOTORS,1	;This way the program knows motor 2 has to work

		MOVLW	B'10010110'	;timerconstants,start value for the timer
		MOVWF	TMR1H		;first 8 bits of TIMER1
		MOVLW	B'00000000'
		MOVWF	TMR1L		;second 8 bits of TIMER1

		BTFSC 	POWERLEVEL,2	
		BSF	PORTB,5		;high current in stepper 2	
		BTFSS 	POWERLEVEL,2
		BCF	PORTB,5		;low current in stepper 2

		BTFSC 	DIRECTION,2
		BSF	PORTB,2		;clockwise turning
		BTFSS 	DIRECTION,2
		BCF	PORTB,2		;counter-clockwise turning

		;Number of loops of 256 steps.
		movlw	B'00000001'
		SUBWF	MOTORSTEPS1,F
		;Number of extra steps
		movlw	B'00000001'
		SUBWF	MOTORSTEPS2,F
		
		MOVF	MOTORSTEPS1,W
		MOVWF	MOTORSTEPS2_1	;write away the MOTORSTEPS1 into the proper variable for stepper2
		MOVF	MOTORSTEPS2,W
		MOVWF	MOTORSTEPS2_2	;write away the MOTORSTEPS2 into the proper variable for stepper2		
		
		MOVF	TMR1H,W
		MOVWF	TIMERDELAY1	;write away the starting time in a temp variable
		MOVF	TIMERDELAY1,W
		MOVWF	TIMERTELLER1	;write away the starting time in the real variable

		MOVLW	B'00000000'
		MOVWF	STEPS2		;steps2 is the number of steps that have to be made still in a loop of 255 steps	
		GOTO	EndInt

;motor_out is ran when the motor stops moving

MOTOR2_OUT	BCF	CHECKMOTORS,1	; motor2 doesn't have to make steps anymore
		BCF	PORTB,5		;make the current low again
		GOTO	EndInt
		
MAKESTEPS_2	MOVLW	D'255'		;put 255 in STEPS2, this is the initialisation for the number of steps to be made still
		MOVWF	STEPS2		;in the loop where 255 steps have to be made.
		RETLW	0


;Now Motor3


;motor_INIT initialises all values for the motor when it has to work

MOTOR3_INIT	BSF	CHECKMOTORS,2	;This way the program knows motor 3 has to work

		MOVLW	B'10000010'	;timerconstants,start value for the timer
		MOVWF	TMR1H		;first 8 bits of TIMER1
		MOVLW	B'00000000'
		MOVWF	TMR1L		;second 8 bits of TIMER1

		BTFSC 	POWERLEVEL,3
		BSF	PORTD,1		;high current in stepper 3	
		BTFSS 	POWERLEVEL,3
		BCF	PORTD,1		;low current in stepper 3

		BTFSC 	DIRECTION,3
		BSF	PORTA,5		;clockwise turning
		BTFSS 	DIRECTION,3
		BCF	PORTA,5		;counter-clockwise turning

		;Number of loops of 256 steps.
		movlw	B'00000001'
		SUBWF	MOTORSTEPS1,F
		;Number of extra steps
		movlw	B'00000001'
		SUBWF	MOTORSTEPS2,F
		
		MOVF	MOTORSTEPS1,W
		MOVWF	MOTORSTEPS3_1	;write away the MOTORSTEPS1 into the proper variable for stepper3
		MOVF	MOTORSTEPS2,W
		MOVWF	MOTORSTEPS3_2	;write away the MOTORSTEPS2 into the proper variable for stepper3	
		
		MOVF	TMR1H,W
		MOVWF	TIMERDELAY3	;write away the starting time in a temp variable
		MOVF	TIMERDELAY3,W
		MOVWF	TIMERTELLER3	;write away the starting time in the real variable

		MOVLW	B'00000000'
		MOVWF	STEPS3		;steps3 is the number of steps that have to be made still in a loop of 255 steps
		GOTO	EndInt

;motor_out is ran when the motor stops moving

MOTOR3_OUT	BCF	CHECKMOTORS,2	; motor3 doesn't have to make steps anymore
		BCF	PORTD,1		;make the current low again
		GOTO	EndInt
		
MAKESTEPS_3	MOVLW	D'255'		;put 255 in STEPS3, this is the initialisation for the number of steps to be made still
		MOVWF	STEPS3		;in the loop where 255 steps have to be made.
		RETLW	0


;MotorLong puts the pulse high
;MotorKort puts the pulse low again
;a combination of motorlong and motorkort gives 1 step
;motorlong also accelerates the interrupt if needed

MOTOR3LONG:	Bank0

		BSF	PORTD,0		;Make pulse high (=clock)

		movlw	B'00000010'	;ADD 2 to the TMR1H=acceleration
		ADDWF	TIMERTELLER3,F	;Save in workregister to check if subtract gives a 0
		MOVLW	B'11100110'	
		SUBWF	TIMERTELLER3,W	;We subtract 230 to see if we ahave 230 for TMR1H
		BTFSC	STATUS,Z	;GOTO next line if TMR1H = 230 (Z = 1)
		GOTO	$+3		 
		movf	TIMERTELLER3,W	;If TMR1H is not 230 we still have to add 2, otherwise we leave TIMERDELAY as it was
		movwf	TIMERDELAY3	;Save subtraction into TIMERDELAY, which is used to determine delay between steps

		MOVF	TIMERDELAY3,W	;write away the timervalues into the proper registers
		MOVWF	TMR1H		;first 8 bits
		MOVLW	B'00000000'
		MOVWF	TMR1L		;second 8 bits

		MOVF	TIMERDELAY3,W
		MOVWF	TIMERTELLER3	;write away to the real variable

		BSF	T2CON,TMR2ON	;turn timer2 on (length of pulse)
		BCF	T1CON,TMR1ON	;turn timer1 off

MOTOR3KORT:	Bank0
		
		BCF	T2CON,TMR2ON	;turn timer2 off, end of pulse
		BSF	T1CON,TMR1ON	;turn timer 1 on (next movement)

		MOVLW	B'00000000'
		SUBWF	STEPS3,F
		BTFSC	STATUS,Z	;check if STEPS3 = 0
		CALL	MAKESTEPS_3	;if so , make STEPS3 = 255 

		MOVLW	B'00000000'
		SUBWF	MOTORSTEPS3_1,F	;check if MOTORSTEPS3_1 = 0
		BTFSS	STATUS,Z
		GOTO 	XLOOP3		;if not then run XLOOP3, this is the number of loops with 255 steps
		GOTO	YLOOP3		;if so, then run YLOOP3, this is the number of extra steps.
		
XLOOP3		BCF	PORTD,0		;pull pulse low (= clock)
		
		DECF	STEPS3,F
		BTFSC	STATUS,Z	;Skip next line if Z = 0, this means STEPS is NOT 0
		DECF	MOTORSTEPS3_1,F	;If STEPS = 0 then we have to subtract one of MOTORSTEPS1 (Z = 1)

;		MOVLW	B'00000011'	;we just write 3 to the pc, to count the steps,
;		MOVWF	TXREG		;this is not done in the final version, it slows down too much	

		MOVLW	B'00000000'
		SUBWF	MOTORSTEPS3_1,W	
		BTFSS	STATUS,Z	;check if motorsteps3_1 = 0
		GOTO	$+5		;if not, the interrupt ends
		MOVLW	B'00000000'
		SUBWF	MOTORSTEPS3_2,W
		BTFSC	STATUS,Z	;if it is, we check if motorsteps3_2 = 0
		GOTO	MOTOR3_OUT	;if it is that means motor3 doesn't have to make anymore moves

		GOTO	EndInt

YLOOP3   	BCF	PORTD,0		;pull pulse low (= clock)
					
;		MOVLW	B'00000011'	;we just write 3 to the pc, to count the steps,
;		MOVWF	TXREG		;this is not done in the final version, it slows down too much	

		DECFSZ	MOTORSTEPS3_2,F	;we have made a step, so the number of steps to make goes down with 1
		GOTO	EndInt		;if not 0 the interrupt ends here

		
		MOVLW	B'11001000'     ;we send 200 when MOTOR3 finishes it's number of steps
		MOVWF	TXREG		;and transmit

		GOTO	MOTOR3_OUT	;motor3 doesn't have to make anymore moves
		GOTO	EndInt	



;Now Motor4


;motor_INIT initialises all values for the motor when it has to work

MOTOR4_INIT	BSF	CHECKMOTORS,3	;This way the program knows motor4 has to work

		MOVLW	B'10111110'	;timerconstants,start value for the timer	
		MOVWF	TMR1H		;first 8 bits of TIMER1
		MOVLW	B'00000000'	
		MOVWF	TMR1L		;second 8 bits of TIMER1

		BTFSC 	POWERLEVEL,4
		BSF	PORTD,7		;high current in stepper 4	
		BTFSS 	POWERLEVEL,4
		BCF	PORTD,7		;low current in stepper 4

		BTFSC 	DIRECTION,4
		BSF	PORTD,6		;clockwise turning
		BTFSS 	DIRECTION,4
		BCF	PORTD,6		;counter-clockwise turning

		;Number of loops of 256 steps.
		movlw	B'00000001'
		SUBWF	MOTORSTEPS1,F
		;Number of extra steps
		movlw	B'00000001'
		SUBWF	MOTORSTEPS2,F
		
		MOVF	MOTORSTEPS1,W
		MOVWF	MOTORSTEPS4_1	;write away the MOTORSTEPS1 into the proper variable for stepper1
		MOVF	MOTORSTEPS2,W
		MOVWF	MOTORSTEPS4_2	;write away the MOTORSTEPS1 into the proper variable for stepper1	
		
		MOVF	TMR1H,W
		MOVWF	TIMERDELAY4	;write away the starting time in a temp variable
		MOVF	TIMERDELAY4,W
		MOVWF	TIMERTELLER4	;write away the starting time in the real variable

		MOVLW	B'00000000'
		MOVWF	STEPS4		;steps4 is the number of steps that have to be made still in a loop of 255 steps	
		GOTO	EndInt

;motor_out is ran when the motor stops moving
	
MOTOR4_OUT	BCF	CHECKMOTORS,3	; motor4 doesn't have to make steps anymore
		BCF	PORTD,7		;make low current again for stepper4
		GOTO	EndInt
		
MAKESTEPS_4	MOVLW	D'255'		;put 255 in STEPS4, this is the initialisation for the number of steps to be made still
		MOVWF	STEPS4		;in the loop where 255 steps have to be made.
		RETLW	0


;MotorLong puts the pulse high
;MotorKort puts the pulse low again
;a combination of motorlong and motorkort gives 1 step
;motorlong also accelerates the interrupt if needed

MOTOR4LONG:	Bank0

		BSF	PORTB,1		;Make pulse high (=clock)

		movlw	B'00000011'	;ADD 3 to the TMR1H =acceleration
		ADDWF	TIMERTELLER4,F	;Save in workregister to check if subtract gives a 0
		MOVLW	B'11111010'	
		SUBWF	TIMERTELLER4,W	;We subtract 250 to see if we ahave 255 for TMR1H = max speed
		BTFSC	STATUS,Z	;GOTO next line if TMR1H = 250 (Z = 1)
		GOTO	$+3		 
		movf	TIMERTELLER4,W	;If TMR1H is not 250 we still have to add 3, otherwise we leave TIMERDELAY as it was
		movwf	TIMERDELAY4	;Save subtraction into TIMERDELAY, which is used to determine delay between steps

		MOVF	TIMERDELAY4,W	;write away the timervalues into the proper registers
		MOVWF	TMR1H		;first 8 bits
		MOVLW	B'00000000'
		MOVWF	TMR1L		;second 8 bits

		MOVF	TIMERDELAY4,W	;write away to the real variable
		MOVWF	TIMERTELLER4

		BSF	T2CON,TMR2ON	;turn timer2 on (length of pulse)
		BCF	T1CON,TMR1ON	;turn timer1 off


MOTOR4KORT:	Bank0
		
		BCF	T2CON,TMR2ON	;turn timer2 off, end of pulse	
		BSF	T1CON,TMR1ON	;turn timer 1 on (next movement)
	
		MOVLW	B'00000000'
		SUBWF	STEPS4,F
		BTFSC	STATUS,Z	;check if STEPS = 0
		CALL	MAKESTEPS_4	;if so , make STEPS1 = 255 	

		MOVLW	B'00000000'
		SUBWF	MOTORSTEPS4_1,F
		BTFSS	STATUS,Z	;check if MOTORSTEPS4_1 = 0
		GOTO 	XLOOP4		;if not then run XLOOP4, this is the number of loops with 255 steps
		GOTO	YLOOP4		;if so, then run YLOOP4, this is the number of extra steps.
		
XLOOP4		BCF	PORTB,1		;pull pulse low (= clock)
		
		DECF	STEPS4,F
		BTFSC	STATUS,Z	;Skip next line if Z = 0, this means STEPS is NOT 0
		DECF	MOTORSTEPS4_1,F	;If STEPS = 0 then we have to subtract one of MOTORSTEPS1 (Z = 1)

;		MOVLW	B'00000100'	;we just write 4 to the pc, to count the steps,
;		MOVWF	TXREG		;this is not done in the final version, it slows down too much	

		MOVLW	B'00000000'
		SUBWF	MOTORSTEPS4_1,W	;check if motorsteps4_1 = 0
		BTFSS	STATUS,Z	;if not the interrupt ends here
		GOTO	$+5
		MOVLW	B'00000000'
		SUBWF	MOTORSTEPS4_2,W	;if it is, we check if motorsteps4_2 = 0
		BTFSC	STATUS,Z
		GOTO	MOTOR4_OUT	;if it is that means motor1 doesn't have to make anymore moves
		
		GOTO	EndInt

YLOOP4   	BCF	PORTB,1		;pull pulse low (= clock)
					
;		MOVLW	B'00000100'	;we just write 3 to the pc, to count the steps,
;		MOVWF	TXREG		;this is not done in the final version, it slows down too much	

		DECFSZ	MOTORSTEPS4_2,F	;we have made a step, so the number of steps to make goes down with 1
		GOTO	EndInt		;if not 0 we check motor
		
		MOVLW	B'11011100'     ;we send 220 when MOTOR4 finishes it's number of steps
		MOVWF	TXREG		;and transmit

		GOTO	MOTOR4_OUT	;motor 4 has made all needed steps, so we run the off function
		GOTO	EndInt		;interrupt ends here


;TimInt and Tim2Int check which motors have to work.
;MotorLong puts the pulse high
;MotorKort puts the pulse low again
;a combination of motorlong and motorkort gives 1 step
;motorlong also accelerates the interrupt if needed

TimInt		BTFSC	CHECKMOTORS,0	;motor1 has to work
		GOTO	MOTOR1LONG	
		BTFSC	CHECKMOTORS,1	;motor2 has to work (same function as motor1 since they have to work at the same time)
		GOTO	MOTOR1LONG
		BTFSC	CHECKMOTORS,2	;motor3 has to work
		GOTO	MOTOR3LONG
		BTFSC	CHECKMOTORS,3	;motor4 has to work
		GOTO	MOTOR4LONG
		GOTO	EndInt

Tim2Int		BTFSC	CHECKMOTORS,0
		GOTO	MOTOR1KORT
		BTFSC	CHECKMOTORS,1
		GOTO	MOTOR1KORT
		BTFSC	CHECKMOTORS,2
		GOTO	MOTOR3KORT
		BTFSC	CHECKMOTORS,3
		GOTO	MOTOR4KORT
		GOTO	EndInt


BlockNotComplete:
		goto	EndInt

;error because OERR overrun error bit is set - clear and continue
ErrOERR:		bcf	RCSTA,CREN	;reset the receiver logic
		bsf	RCSTA,CREN	;enable reception again
		goto	EndInt
;error because FERR framing error bit is set - clear and continue
ErrFERR:	movf	RCREG,W		;discard received data that has error
		goto	EndInt

;******************************************************************************
;Read analog data and transmit the data.
PutData:
		GOTO	EndInt


		END